@@ -10,6 +10,16 @@
 #' @keywords internal
 RowKronecker <- function(X1,
                          X2) {
+  if (isTRUE(class(X1) == 'spam') & isTRUE(class(X2) == 'spam')) {
+    L <- RowKron(X1, X2)
+    rowKron <- spam::spam(x=0, nrow=0,ncol=0)
+    rowKron@entries <- L$entries
+    rowKron@colindices <- L$colindices
+    rowKron@rowpointers <- L$rowpointers
+    rowKron@dimension <- L$dimension
+    return(rowKron)
+  }
+
   one.1 <- matrix(1, 1, ncol(X1))
   one.2 <- matrix(1, 1, ncol(X2))
   rowKron <- kronecker(X1, one.2) * kronecker(one.1, X2)
